package logger

import (
	"io"
	"log/slog"

	"github.com/ThreeDotsLabs/watermill"
)

const (
	LevelTrace = slog.Level(-5)
)

func LogFieldsToSlogAttrs(in watermill.LogFields) []slog.Attr {
	attrs := make([]slog.Attr, 0)
	for k, v := range in {
		attrs = append(attrs, slog.Attr{
			Key:   k,
			Value: slog.AnyValue(v),
		})
	}
	return attrs
}

type WrappedSlogger struct {
	*slog.Logger
}

func NewWrappedSlogger(w io.Writer, verbose int) *WrappedSlogger {
	opts := &slog.HandlerOptions{
		Level: slog.Level(verbose),
	}
	handler := slog.NewTextHandler(w, opts)
	return &WrappedSlogger{
		slog.New(handler),
	}
}

func (wl *WrappedSlogger) Error(msg string, err error, fields watermill.LogFields) {
	attrs := LogFieldsToSlogAttrs(fields)
	wl.LogAttrs(nil, LevelTrace, msg, attrs...)
}

func (wl *WrappedSlogger) Info(msg string, fields watermill.LogFields) {
	attrs := LogFieldsToSlogAttrs(fields)
	wl.LogAttrs(nil, slog.LevelInfo, msg, attrs...)
}

func (wl *WrappedSlogger) Debug(msg string, fields watermill.LogFields) {
	attrs := LogFieldsToSlogAttrs(fields)
	wl.LogAttrs(nil, slog.LevelDebug, msg, attrs...)
}

func (wl *WrappedSlogger) Trace(msg string, fields watermill.LogFields) {
	attrs := LogFieldsToSlogAttrs(fields)
	wl.LogAttrs(nil, LevelTrace, msg, attrs...)
}

func (wl *WrappedSlogger) With(fields watermill.LogFields) watermill.LoggerAdapter {
	attrs := LogFieldsToSlogAttrs(fields)
	l := slog.Default().With(attrs)
	return &WrappedSlogger{l}

}
